#ifndef MODELH
#define MODELH

#include <d3dx9.h>
#include <wchar.h>
#include <Coef.h>
#include <string>
#include <map>

#include <OrgWindow.h>

using namespace std;

class CMQOMaterial;
class CMQOObject;
class CMQOFace;
class CBone;
class CInfScope;
class CMySprite;
class CMotionPoint;
class CQuaternion;
class CKbsFile;
class CRps;

typedef struct tag_newmpelem
{
	CBone* boneptr;
	CMotionPoint* mpptr;
}NEWMPELEM;


class CModel
{
public:
	CModel();
	~CModel();

	int LoadMQO( LPDIRECT3DDEVICE9 pdev, WCHAR* wfile, WCHAR* modelfolder, float srcmult, int ismedia );
	int OnRender( LPDIRECT3DDEVICE9 pdev, int lightflag, D3DXVECTOR4 diffusemult );
	int RenderBoneMark( LPDIRECT3DDEVICE9 pdev, CModel* bmarkptr, CMySprite* bcircleptr, int selboneno );
	int GetModelBound( MODELBOUND* dstb );

	int MakeObjectName();
	int MakeBoneTri();
	int MakeInfScope();
	int MakePolyMesh3();
	int MakeExtLine();
	int MakeDispObj();

	int CalcInf();

	int DbgDump();

	int UpdateMatrix( int startbone, D3DXMATRIX* wmat, D3DXMATRIX* vpmat );
	int SetShaderConst();

	int SetBoneEul( int boneno, D3DXVECTOR3 srceul );
	int GetBoneEul( int boneno, D3DXVECTOR3* dsteul );

	int FillTimeLine( OrgWinGUI::OWP_Timeline& timeline, map<int, int>& lineno2boneno, map<int, int>& boneno2lineno );

	int AddMotion( char* srcname, WCHAR* wfilename, double srcleng, int* dstid );
	CMotionPoint* AddMotionPoint( int srcmotid, double srcframe, int srcboneno, int calcflag, int* existptr );

	int SetCurrentMotion( int srcmotid );
	int SetMotionFrame( double srcframe );
	int GetMotionFrame( double* dstframe );
	int SetMotionSpeed( double srcspeed );
	int GetMotionSpeed( double* dstspeed );

	int DeleteMotion( int motid );
	int GetSymBoneNo( int srcboneno, int* dstboneno, int* existptr );

	int PickBone( PICKINFO* pickinfo );
	int IKRotate( int srcboneno, D3DXVECTOR3 targetpos, int maxlevel );
	int IKRotateAxisDelta( int axiskind, int srcboneno, float delta, int maxlevel );
	int IKMove( int srcboneno, D3DXVECTOR3 targetpos );
	int IKMoveAxisDelta( int axiskind, int srcboneno, float deltax );

	int CollisionNoBoneObj_Mouse( PICKINFO* pickinfo, char* objnameptr );
	int TransformBone( int winx, int winy, int srcboneno, D3DXVECTOR3* worldptr, D3DXVECTOR3* screenptr, D3DXVECTOR3* dispptr );
	int ChangeMotFrameLeng( int motid, double srcleng );
	int AdvanceTime( int previewflag, double difftime, double* nextframeptr, int* endflagptr );

	int SaveQubFile( WCHAR* savepath, int savemotid );
	int LoadQubFile( WCHAR* loadpath, int* newmotid );

	int MakeEnglishName();
	int CalcXTransformMatrix( float mult );
	int AddDefMaterial();

	int SetMiko3rdVec( int boneno, D3DXVECTOR3 srcvec );
	int UpdateMatrixNext();
	int SetKinectFirstPose();
private:
	int InitParams();
	int DestroyObjs();
	int CreateMaterialTexture();

	int AddModelBound( MODELBOUND* mb, MODELBOUND* addmb );

	int DestroyMaterial();
	int DestroyObject();
	int DestroyAncObj();
	int DestroyAllMotionInfo();

	void MakeBoneReq( CBone* parbone, CMQOFace* curface, D3DXVECTOR3* pointptr, int broflag, int* errcntptr );

	int SetShapeNoReq( CMQOFace** ppface, int facenum, int searchp, int shapeno, int* setfacenum);
	int SetFaceOfShape( CMQOFace** ppface, int facenum, int shapeno, CMQOFace** ppface2, int setfacenum );

	int DbgDumpBoneReq( CBone* boneptr, int broflag );

	void UpdateMatrixReq( int srcmotid, double srcframe, D3DXMATRIX* wmat, D3DXMATRIX* vpmat, 
		D3DXMATRIX* parmat, CQuaternion* parq, CBone* srcbone, int broflag );

	void FillTimelineReq( OrgWinGUI::OWP_Timeline& timeline, CBone* curbone, int* linenoptr, 
		map<int, int>& lineno2boneno, map<int, int>& boneno2lineno, int broflag );

	void SetSelectFlagReq( CBone* boneptr, int broflag );
	int CalcMouseLocalRay( PICKINFO* pickinfo, D3DXVECTOR3* startptr, D3DXVECTOR3* dirptr );
	CBone* GetCalcRootBone( CBone* firstbone, int maxlevel );
	void CalcXTransformMatrixReq( CBone* srcbone, D3DXMATRIX parenttra, float mult );

public:
	int m_modelno;

	LPDIRECT3DDEVICE9 m_pdev;

	bool m_modeldisp;
	map<int, CMQOMaterial*> m_material;
	map<int, CMQOObject*> m_object;
	map<string, CMQOObject*> m_objectname;

	map<int, CMQOMaterial*> m_ancmaterial;
	map<int, CMQOObject*> m_ancobject;
	map<int, CMQOObject*> m_boneobject;
	map<string, CBone*> m_bonename;

	map<int, CBone*> m_bonelist;

	//int m_topbonenum;
	CBone* m_topbone;

	map<int, CInfScope*> m_infscope;


	WCHAR m_filename[MAX_PATH];
	WCHAR m_dirname[MAX_PATH];
	WCHAR m_modelfolder[MAX_PATH];

	float m_loadmult;

	D3DXMATRIX m_bonemat[ MAXBONENUM ];

	D3DXMATRIX m_matWorld;
	D3DXMATRIX m_matVP;

	map<int, MOTINFO*> m_motinfo;
	MOTINFO* m_curmotinfo;

	vector<NEWMPELEM> m_newmplist;

	CKbsFile* m_kbs;
	CRps* m_rps;

};

#endif